"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Agent = exports.toHttpResponse = exports.toHeader = exports.parseQuery = void 0;
const core_interface_1 = require("@ideal-postcodes/core-interface");
const timed_fetch_1 = require("./timed_fetch");
/**
 * @hidden
 */
const { IdealPostcodesError } = core_interface_1.errors;
/**
 * Rewrites `key` `value` to uri encoded query string component
 *
 * @hidden
 */
const toParam = (key, value) => `${encodeURIComponent(key)}=${encodeURIComponent(value)}`;
/**
 * @hidden
 */
const parseQuery = (query) => {
    const keys = Object.keys(query);
    if (keys.length === 0)
        return "";
    return "?" + keys.map((key) => toParam(key, query[key])).join("&");
};
exports.parseQuery = parseQuery;
/**
 * @hidden
 */
const toHeader = (headers) => {
    const result = {};
    headers.forEach((value, key) => (result[key] = value));
    return result;
};
exports.toHeader = toHeader;
/**
 * Adapts fetch response to one that can be used by client
 *
 * @hidden
 */
const toHttpResponse = (httpRequest, response, body) => ({
    httpRequest,
    body,
    httpStatus: response.status,
    header: (0, exports.toHeader)(response.headers),
    metadata: { response },
});
exports.toHttpResponse = toHttpResponse;
/**
 * Wraps non-API error in IdealPostcodesError with status code of 0
 *
 * @hidden
 */
const handleError = (error) => {
    const idpcError = new IdealPostcodesError({
        message: `[${error.name}] ${error.message}`,
        httpStatus: 0,
        metadata: { fetch: error },
    });
    return Promise.reject(idpcError);
};
/**
 * Implements browser agent for core-interface client
 *
 * @hidden
 */
class Agent {
    constructor(config = {}) {
        this.config = config;
        /**
         * Default config using RequestInit interface from DOM specification
         */
        this.defaultConfig = {
            // The cache mode you want to use for the request
            cache: "no-cache",
            // The request credentials you want to use for the request: omit, same-origin, or include. The default is same-origin
            credentials: "omit",
            // The mode you want to use for the request, e.g., cors, no-cors, same-origin, or navigate. The default is cors
            mode: "cors",
            // redirect: The redirect mode to use: follow, error, or manual. The default is follow
            redirect: "follow",
        };
    }
    // Return AbortController instance if present
    abortController() {
        if (window.AbortController === undefined)
            return;
        return new window.AbortController();
    }
    http(httpRequest) {
        const { body, method, url, header, query, timeout } = httpRequest;
        const requestInit = {
            method,
            headers: header,
            ...this.defaultConfig,
            ...this.config,
        };
        const abortController = this.abortController();
        if (abortController)
            requestInit.signal = abortController.signal;
        try {
            if (body !== undefined)
                requestInit.body = JSON.stringify(body);
        }
        catch (error) {
            return handleError(error);
        }
        const uri = `${url}${(0, exports.parseQuery)(query)}`;
        let response;
        return (0, timed_fetch_1.timedFetch)(uri, requestInit, timeout, abortController)
            .then((r) => {
            response = r;
            return r.json();
        })
            .then((responseBody) => (0, exports.toHttpResponse)(httpRequest, response, responseBody))
            .catch(handleError);
    }
}
exports.Agent = Agent;
